//
//  MainTabViewController.swift
//  SampleApp
//
//  Created by Nishan Niraula on 4/27/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import UIKit
import SwiftUI
import Realm
import AmitySDK

/*
 This class sets up various screens in sample app.
 */
class MainTabViewController: UITabBarController {
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Note:
        // Simple check to catch any fatal exception thrown by realm model from sdk.
        let realm = RLMRealm.default()
        Log.add(info: "Realm Classes: \(String(describing: realm.configuration.objectClasses))")
        
        setupTabControllers()
    }
    
    func setupTabControllers() {
        // Our sdk also supports push notification. We regisiter for push notification here
        let client = AmityManager.shared.client!
        
        // Home Screen
        let homeStoryboard = UIStoryboard(name: "Home", bundle: nil)
        let homeController = homeStoryboard.instantiateViewController(withIdentifier: "HomeViewController") as! HomeViewController
        homeController.tabBarItem = UITabBarItem(title: "Chat", image: UIImage(named: "tab_chat"), tag: 0)
        homeController.channelType = .standard
        
        // Feed Screen
        let postsFeedStoryboard = UIStoryboard(name: "Feed", bundle: nil)
        
        // Post Selection Screen
        let postListController = ListViewController()
        postListController.listTitle = "Feed"
        postListController.items = FeedMenu.allCases.map{ ListItem(id: $0.rawValue, title: $0.title) }
        postListController.listItemTapAction = { item, index in
            guard let menu = FeedMenu(rawValue: item.id) else {
                assertionFailure("Unhandled case")
                return
            }
            switch menu {
            case .userFeed, .globalFeed, .customPostRankingGlobalFeed:
                let postsFeedController = postsFeedStoryboard.instantiateViewController(withIdentifier: UserPostsFeedViewController.identifier) as! UserPostsFeedViewController
                let feedManager = UserPostsFeedManager(client: client, userId: nil)
                postsFeedController.feedManager = feedManager
                postsFeedController.title = item.title
                
                if menu == .userFeed {
                    postsFeedController.feedManager.feedType = .myFeed
                } else if menu == .globalFeed {
                    postsFeedController.feedManager.feedType = .globalFeed
                } else {
                    postsFeedController.feedManager.feedType = .customPostRankingGlobalFeed
                }
                
                postListController.navigationController?.pushViewController(postsFeedController, animated: true)
            case .createComment:
                let controller = UIHostingController(rootView: CommentCreateTestView())
                postListController.navigationController?.pushViewController(controller, animated: true)
            case .getLatestComments:
                
                    let controller = UIHostingController(rootView: LatestCommentView())
                    postListController.navigationController?.pushViewController(controller, animated: true)
            case .getPosts:
                if #available(iOS 14.0, *) {
                    let postQuerySettingsPage = PostQuerySettingsPage()
                    let vc = UIHostingController(rootView: postQuerySettingsPage)
                    postListController.navigationController?.pushViewController(vc, animated: true)
                } else {
                    assertionFailure("Unhandled case")
                }
            case .singlePost:
                let singlePostView = SinglePostQueryView()
                let vc = UIHostingController(rootView: singlePostView)
                postListController.navigationController?.pushViewController(vc, animated: true)
                
            case .getFile:
                let getFileView = GetFileView()
                let vc = UIHostingController(rootView: getFileView)
                postListController.navigationController?.pushViewController(vc, animated: true)
            
            case .getCategory:
                let getCategoryView = GetCategoryView()
                let vc = UIHostingController(rootView: getCategoryView)
                postListController.navigationController?.pushViewController(vc, animated: true)
                
            case .getComment:
                let getCommentView = GetCommentView()
                let vc = UIHostingController(rootView: getCommentView)
                postListController.navigationController?.pushViewController(vc, animated: true)
            case .getStory:
                let getStoryView = GetStoryView()
                let vc = UIHostingController(rootView: getStoryView)
                postListController.navigationController?.pushViewController(vc, animated: true)
            case .storyTargetGlobalFeed:
                let controller = UIHostingController(rootView: GetGlobalFeedStoryTargetsView(viewModel: StoryTargetListViewModel()).environment(\.navigationController, postListController.navigationController))
                postListController.navigationController?.pushViewController(controller, animated: true)
            case .verifyView:
                let controller = UIHostingController(rootView: VerifyURLView(viewModel: VerifyURLViewModel()).environment(\.navigationController, postListController.navigationController))
                postListController.navigationController?.pushViewController(controller, animated: true)
            case .networkAds:
                let controller = UIHostingController(rootView: NetworkAdsListView().environment(\.navigationController, postListController.navigationController))
                postListController.navigationController?.pushViewController(controller, animated: true)
            case .networkActivities:
                let controller = UIHostingController(rootView: NetworkActivityView())
                postListController.present(controller, animated: true)
            case .getPostsByIds:
                let controller = UIHostingController(rootView: MultiplePostQueryView())
                postListController.navigationController?.pushViewController(controller, animated: true)
            case .postSemanticSearch:
                let controller = UIHostingController(rootView: PostSemanticSearchView())
                postListController.navigationController?.pushViewController(controller, animated: true)
            }
        }
        postListController.tabBarItem = UITabBarItem(title: "My Feed", image: UIImage(named: "tab_feed"), tag: 1)
        
        // User Selection Screen
        let userListOptionController = ListViewController()
        userListOptionController.listTitle = "Users"
        userListOptionController.items = UserMenu.allCases.map{ ListItem(id: $0.rawValue, title: $0.title)}
        userListOptionController.listItemTapAction = { item, index in
            guard let menu = UserMenu(rawValue: item.id) else {
                assertionFailure("Unhandled case")
                return
            }
            
            switch menu {
            case .userList:
                let userListController = postsFeedStoryboard.instantiateViewController(withIdentifier: UserListViewController.identifier) as! UserListViewController
                userListOptionController.navigationController?.pushViewController(userListController, animated: true)
                
            case .singleUser:
                // Show default user view
                let individualUserController = UIHostingController(rootView: UserQueryView())
                userListOptionController.navigationController?.pushViewController(individualUserController, animated: true)
                
            case .permission:
                let permissionController = UIHostingController(rootView: UserPermissionView(channelId: nil))
                userListOptionController.navigationController?.pushViewController(permissionController, animated: true)
                
            case .followingList:
                let followingListController = postsFeedStoryboard.instantiateViewController(withIdentifier: FollowListViewController.identifier) as! FollowListViewController
                followingListController.pageType = .following
                userListOptionController.navigationController?.pushViewController(followingListController, animated: true)
                
            case .followerList:
                let followingListController = postsFeedStoryboard.instantiateViewController(withIdentifier: FollowListViewController.identifier) as! FollowListViewController
                followingListController.pageType = .follower
                userListOptionController.navigationController?.pushViewController(followingListController, animated: true)
            case .followRTE:
                let controller = MyFollowRealTimeEventController()
                let manager = MyFollowRealTimeEventManager()
                controller.manager = manager

                userListOptionController.navigationController?.pushViewController(controller, animated: true)
            case .blockedUserList:
                let controller = BlockedUserListViewController()
                userListOptionController.navigationController?.pushViewController(controller, animated: true)
                
            case .presenceSettings:
                let controller = PresenceSettingsViewController()
                userListOptionController.navigationController?.pushViewController(controller, animated: true)
                
            case .onlineUsers:
                let controller = OnlineUsersViewController()
                userListOptionController.navigationController?.pushViewController(controller, animated: true)
                
            case .updateMetadata:
                let host = UIHostingController(rootView: MetadataInputView(buttonAction: { metadata in
                    // Update user metadata
                    UserMetadataUpdater().updateMetadata(metadata: metadata, vc: self)
                }))
                userListOptionController.navigationController?.pushViewController(host, animated: true)
            }
        }
        userListOptionController.tabBarItem = UITabBarItem(title: "User List", image: UIImage(named: "ic_tab_users"), tag: 2)
        
        // Community Selection Screen
        let communitiesListController = ListViewController()
        communitiesListController.listTitle = "Communities"
        communitiesListController.items = ["Default","Trending", "Recommended", "Single Community", "Stories by targets", "Story Targets by targets", "Community Semantic Search"].map{ ListItem(id: $0, title: $0)}
        communitiesListController.listItemTapAction = { item, index in
            
            var communityType: CommunityType = .normal
            switch index {
            case 0:
                communityType = .normal
            case 1:
                communityType = .trending
            case 2:
                communityType = .recommended
            case 3:
                communityType = .normal
                
                let controller = UIHostingController(rootView: SingleCommunityView().environment(\.navigationController, communitiesListController.navigationController))
                communitiesListController.navigationController?.pushViewController(controller, animated: true)
                
                return
            case 4:
                let controller = UIHostingController(rootView: StoryByTargetsListView(viewModel: StoryListViewModel()).environment(\.navigationController, communitiesListController.navigationController))
                communitiesListController.navigationController?.pushViewController(controller, animated: true)
                return
            case 5:
                let controller = UIHostingController(rootView: GetStoryTargetsView(viewModel: StoryTargetListViewModel()).environment(\.navigationController, communitiesListController.navigationController))
                communitiesListController.navigationController?.pushViewController(controller, animated: true)
                return
            case 6:
                let controller = UIHostingController(rootView: CommunitySemanticSearchView())
                communitiesListController.navigationController?.pushViewController(controller, animated: true)
                return
            default:
                break
            }
            
            let communityController = UIHostingController(rootView: CommunityView(viewModel: CommunityListViewModel(type: communityType)).environment(\.navigationController, communitiesListController.navigationController))
            communitiesListController.navigationController?.pushViewController(communityController, animated: true)
        }
        communitiesListController.tabBarItem = UITabBarItem(title: "Communities", image: UIImage(named: "tab_comm"), tag: 3)
        
        var controllers = [homeController, postListController, userListOptionController, communitiesListController]
        
        let testController = TestController()
        testController.tabBarItem = UITabBarItem(title: "Test", image: UIImage(systemName: "note"), tag: 4)
        controllers.append(testController)
        
        // Tabs
        controllers = controllers.map{
            
            let navController = UINavigationController(rootViewController: $0)
            navController.tabBarItem = $0.tabBarItem
            return navController
        }
        
        viewControllers = controllers
    }
}

struct SwiftUIMainTabView: UIViewControllerRepresentable {
    
    typealias UIViewControllerType = MainTabViewController
    
    func makeUIViewController(context: Context) -> MainTabViewController {
        let dashboardController = MainTabViewController()
        dashboardController.modalPresentationStyle = .fullScreen
        
        return dashboardController
    }
    
    func updateUIViewController(_ uiViewController: MainTabViewController, context: Context) {
        // Left empty
    }
}

extension MainTabViewController {
    
    private enum FeedMenu: String, CaseIterable {
        case userFeed
        case globalFeed
        case customPostRankingGlobalFeed
        case postSemanticSearch
        case createComment
        case getLatestComments
        case getPosts
        case getPostsByIds
        case singlePost
        case getFile
        case getCategory
        case getComment
        case getStory
        case storyTargetGlobalFeed
        case verifyView
        case networkAds
        case networkActivities
        
        var title: String {
            switch self {
            case .userFeed:
                return "User Feed"
            case .globalFeed:
                return "Global Feed"
            case .customPostRankingGlobalFeed:
                return "Custom Post Ranking Global Feed"
            case .createComment:
                return "Create Comment"
            case .getLatestComments:
                return "Get Latest Comment"
            case .getPosts:
                return "Get Posts"
            case .singlePost:
                return "Query Single Post"
            case .getFile:
                return "Get File"
            case .getCategory:
                return "Get Category"
            case .getComment:
                return "Get Comment"
            case .getStory:
                return "Get Story"
            case .storyTargetGlobalFeed:
                return "Story Target Global Feed"
            case .verifyView:
                return "Verify white list"
            case .networkAds:
                return "Network Ads"
            case .networkActivities:
                return "Network Activities"
            case .getPostsByIds:
                return "Get Posts By Ids"
            case .postSemanticSearch:
                return "Semantic Search - Post"
            }
        }
    }
    
    private enum UserMenu: String, CaseIterable {
        case userList
        case singleUser
        case permission
        case followingList
        case followerList
        case followRTE
        case blockedUserList
        case presenceSettings
        case onlineUsers
        case updateMetadata
        
        var title: String {
            switch self {
            case .userList:
                return "User List"
            case .singleUser:
                return "Individual User"
            case .permission:
                return "Check Permission"
            case .followingList:
                return "My Following List"
            case .followerList:
                return "My Follower List"
            case .followRTE:
                return "My Follow RTE"
            case .blockedUserList:
                return "Blocked User List"
            case .presenceSettings:
                return "Presence Settings"
            case .onlineUsers:
                return "Online Users"
            case .updateMetadata:
                return "Update My Metadata"
            }
        }
    }
}

struct UserMetadataUpdater {
    
    @MainActor
    func updateMetadata(metadata: [(String, String)], vc: UIViewController) {
        guard !metadata.isEmpty else { return }
        
        var finalMetadata = [String: String]()
        metadata.forEach {
            finalMetadata[$0.0] = $0.1
        }
        
        let updateBuilder = AmityUserUpdateBuilder()
        updateBuilder.setUserMetadata(finalMetadata)
        
        Task { @MainActor in
            do {
                try await AmityManager.shared.client?.editUser(updateBuilder)
                
                AppUtility.showAlert(in: vc, title: "", message: "Metadata Updated!")
            } catch let error {
                AppUtility.showAlert(in: vc, title: "Error", message: error.localizedDescription)
            }
        }
    }
}
